var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import fse from "fs-extra";
import * as path from "node:path";
import Relation from "./core/Relation.js";
import { relationId } from "./core/relationId.js";
import createMarkdownRelations from "./mdx/createRelations.js";
export default class {
    constructor(cwd) {
        this.cwd = cwd || process.cwd();
        this.workingDirectory = this.cwd;
        this.relationFilePath = path.join(this.workingDirectory, ".relation", "relation.json");
    }
    checkIsInit() {
        if (fse.existsSync(this.relationFilePath)) {
            return true;
        }
        return false;
    }
    init() {
        if (this.checkIsInit()) {
            return true;
        }
        fse.ensureFileSync(this.relationFilePath);
        fse.writeFileSync(this.relationFilePath, "[]");
        return true;
    }
    read() {
        this.init();
        const relationBuffer = fse.readFileSync(this.relationFilePath);
        const rawRelations = JSON.parse(relationBuffer.toString());
        const relation = rawRelations.map((d) => {
            return new Relation(Object.assign({ workingDirectory: this.workingDirectory }, d));
        });
        return relation;
    }
    write(relations) {
        return fse.writeFileSync(this.relationFilePath, JSON.stringify(relations, null, 2));
    }
    createMarkdownRelations(info) {
        return __awaiter(this, void 0, void 0, function* () {
            const newRelations = yield createMarkdownRelations(Object.assign({ workingDirectory: this.workingDirectory }, info));
            return newRelations;
        });
    }
    create(info) {
        return __awaiter(this, void 0, void 0, function* () {
            const relation = new Relation({
                workingDirectory: this.workingDirectory,
                id: relationId(),
                fromRange: info.fromRange,
                toRange: info.toRange,
            });
            relation.fromAbsolutePath = info.fromAbsolutePath;
            relation.toAbsolutePath = info.toAbsolutePath;
            yield relation.autoSetFromRev();
            yield relation.autoSetToRev();
            return relation;
        });
    }
    filter(filter) {
        const rawRelations = this.read();
        const relations = rawRelations.filter(filter);
        return relations;
    }
    updateById(id, update) {
        return __awaiter(this, void 0, void 0, function* () {
            const relations = this.read();
            const newRelations = [];
            for (const relation of relations) {
                if (relation.id === id) {
                    newRelations.push(yield update(relation));
                    continue;
                }
                newRelations.push(relation);
            }
            this.write(newRelations);
        });
    }
    getRelationViewerData(relations) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!Array.isArray(relations)) {
                return;
            }
            const fromModifiedContent = relations[0].fromCurrentContent;
            const toModifiedContent = relations[0].toCurrentContent;
            const viewerRelations = [];
            const viewerContents = {};
            for (const relation of relations) {
                const [viewerRelation, viewerContent,] = yield relation.getViewerRelationAndContents();
                viewerRelations.push(viewerRelation);
                Object.assign(viewerContents, viewerContent);
            }
            const relationViewerData = {
                fromPath: relations[0].fromPath,
                toPath: relations[0].toPath,
                fromModifiedContent,
                toModifiedContent,
                viewerRelations: viewerRelations,
                viewerContents: viewerContents,
            };
            return relationViewerData;
        });
    }
}
