var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import path from "path";
import simpleGit from "simple-git";
let singleton = null;
export default class GitServer {
    constructor() {
        this.showMap = new Map();
        this.revMap = new Map();
    }
    show(workingDirectory, parsedRevision, gitWorkingDirectory = "", filePath) {
        return __awaiter(this, void 0, void 0, function* () {
            const key = `${parsedRevision}:${filePath}`;
            let result = this.showMap.get(key);
            if (result !== undefined) {
                return result;
            }
            const simpleGitInstance = simpleGit(path.join(workingDirectory, gitWorkingDirectory));
            try {
                result = yield simpleGitInstance.show([
                    `${parsedRevision}:./${filePath}`,
                ]);
            }
            catch (error) {
                console.error(error);
                result = "";
            }
            if (parsedRevision !== ":0") {
                this.showMap.set(key, result);
            }
            return result;
        });
    }
    static parseRev(revision, absoluteGitWorkingDirectory) {
        return __awaiter(this, void 0, void 0, function* () {
            if (revision === undefined) {
                return revision;
            }
            const simpleGitInstance = simpleGit(absoluteGitWorkingDirectory);
            const result = (yield simpleGitInstance.raw("rev-parse", revision)).trim();
            return result;
        });
    }
    static singleton() {
        if (!singleton) {
            singleton = new GitServer();
        }
        return singleton;
    }
}
