var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import fse from "fs-extra";
import * as path from "node:path";
import simpleGit from "simple-git";
import ChangeServer from "./ChangeServer.js";
import { checkDirty } from "./checkDirty.js";
import getDirnameBasename from "./getDirnameBasename.js";
import GitServer from "./GitServer.js";
import { sha1 } from "./sha1.js";
export var RelationRevType;
(function (RelationRevType) {
    RelationRevType["content"] = "content";
    RelationRevType["git"] = "git";
})(RelationRevType || (RelationRevType = {}));
class Relation {
    constructor(relationInfo = {}) {
        this.id = undefined;
        this.fromRange = undefined;
        this.toRange = undefined;
        this.fromPath = undefined;
        this.toPath = undefined;
        this.fromContentRev = undefined;
        this.toContentRev = undefined;
        this.fromGitRev = undefined;
        this.toGitRev = undefined;
        this.fromGitWorkingDirectory = undefined;
        this.toGitWorkingDirectory = undefined;
        this.workingDirectory = undefined;
        this.id = relationInfo.id;
        this.fromRange = relationInfo.fromRange;
        this.toRange = relationInfo.toRange;
        this.fromPath = relationInfo.fromPath;
        this.toPath = relationInfo.toPath;
        this.fromContentRev = relationInfo.fromContentRev;
        this.toContentRev = relationInfo.toContentRev;
        this.fromGitRev = relationInfo.fromGitRev;
        this.toGitRev = relationInfo.toGitRev;
        this.fromGitWorkingDirectory = relationInfo.fromGitWorkingDirectory;
        this.toGitWorkingDirectory = relationInfo.toGitWorkingDirectory;
        this.workingDirectory = relationInfo.workingDirectory;
    }
    get fromRevType() {
        if (this.fromGitRev) {
            return RelationRevType.git;
        }
        if (this.fromContentRev) {
            return RelationRevType.content;
        }
    }
    get toRevType() {
        if (this.toGitRev) {
            return RelationRevType.git;
        }
        if (this.toContentRev) {
            return RelationRevType.content;
        }
    }
    get fromAbsolutePath() {
        const absolutePath = this.getAbsolutePath(this.fromPath);
        return absolutePath;
    }
    set fromAbsolutePath(absolutePath) {
        this.fromPath = this.getRelativePath(absolutePath);
        this.fromGitWorkingDirectory = undefined;
    }
    get toAbsolutePath() {
        const absolutePath = this.getAbsolutePath(this.toPath);
        return absolutePath;
    }
    set toAbsolutePath(absolutePath) {
        this.toPath = this.getRelativePath(absolutePath);
        this.toGitWorkingDirectory = undefined;
    }
    get fromGitWorkingDirectoryString() {
        if (!this.fromGitWorkingDirectory) {
            return "";
        }
        return this.fromGitWorkingDirectory;
    }
    get toGitWorkingDirectoryString() {
        if (!this.toGitWorkingDirectory) {
            return "";
        }
        return this.toGitWorkingDirectory;
    }
    get fromAbsoluteGitWorkingDirectory() {
        const absolutePath = this.getAbsolutePath(this.fromGitWorkingDirectoryString);
        return absolutePath;
    }
    get toAbsoluteGitWorkingDirectory() {
        const absolutePath = this.getAbsolutePath(this.toGitWorkingDirectoryString);
        return absolutePath;
    }
    get fromRelativeGitPath() {
        const relativePath = path.relative(this.fromAbsoluteGitWorkingDirectory, this.fromAbsolutePath);
        return relativePath;
    }
    get toRelativeGitPath() {
        const relativePath = path.relative(this.toAbsoluteGitWorkingDirectory, this.toAbsolutePath);
        return relativePath;
    }
    get fromCurrentContent() {
        const content = fse.readFileSync(this.fromAbsolutePath).toString();
        return content;
    }
    get toCurrentContent() {
        const content = fse.readFileSync(this.toAbsolutePath).toString();
        return content;
    }
    getFromDirty() {
        return __awaiter(this, void 0, void 0, function* () {
            const originalContent = yield this.getFromOriginalContent();
            const currentContent = this.fromCurrentContent;
            const changes = ChangeServer.singleton().getFixedChanges(originalContent, currentContent);
            const dirty = checkDirty({
                changes: changes,
                range: this.fromRange,
            });
            return dirty;
        });
    }
    getFromGitContent(rev) {
        return __awaiter(this, void 0, void 0, function* () {
            const absolutePath = this.fromAbsolutePath;
            const absoluteGitWorkingDirectory = yield this.getAbsoluteGitWorkingDirectory(absolutePath);
            const content = yield this.getGitContent({
                rev,
                absoluteGitWorkingDirectory,
                relativeGitPath: path.relative(absoluteGitWorkingDirectory, absolutePath),
                safe: true,
            });
            return content;
        });
    }
    getToGitContent(rev) {
        return __awaiter(this, void 0, void 0, function* () {
            const absolutePath = this.toAbsolutePath;
            const absoluteGitWorkingDirectory = yield this.getAbsoluteGitWorkingDirectory(absolutePath);
            const content = yield this.getGitContent({
                rev,
                absoluteGitWorkingDirectory,
                relativeGitPath: path.relative(absoluteGitWorkingDirectory, absolutePath),
                safe: true,
            });
            return content;
        });
    }
    getFromOriginalContent() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.fromRevType === RelationRevType.git) {
                const content = yield this.getGitContent({
                    rev: this.fromGitRev,
                    absoluteGitWorkingDirectory: this.fromAbsoluteGitWorkingDirectory,
                    relativeGitPath: this.fromRelativeGitPath,
                    safe: true,
                });
                return content;
            }
            if (this.fromRevType === RelationRevType.content) {
                const content = this.getContentSafe(this.fromContentRev);
                return content;
            }
        });
    }
    getToOriginalContent() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.toRevType === RelationRevType.git) {
                const content = yield this.getGitContent({
                    rev: this.toGitRev,
                    absoluteGitWorkingDirectory: this.toAbsoluteGitWorkingDirectory,
                    relativeGitPath: this.toRelativeGitPath,
                    safe: true,
                });
                return content;
            }
            if (this.toRevType === RelationRevType.content) {
                const content = this.getContentSafe(this.toContentRev);
                return content;
            }
        });
    }
    getContent(contentRev) {
        const content = fse
            .readFileSync(path.join(this.workingDirectory, ".relation", "contents", contentRev))
            .toString();
        return content;
    }
    getContentSafe(contentRev) {
        let content = "";
        try {
            content = this.getContent(contentRev);
        }
        catch (error) {
            console.error(error);
        }
        return content;
    }
    getGitContent({ absoluteGitWorkingDirectory, rev, relativeGitPath, safe, }) {
        return __awaiter(this, void 0, void 0, function* () {
            let content = "";
            try {
                const simpleGitInstance = simpleGit(absoluteGitWorkingDirectory);
                content = yield simpleGitInstance.show([`${rev}:${relativeGitPath}`]);
            }
            catch (error) {
                if (!safe) {
                    throw error;
                }
                console.error(error);
            }
            return content;
        });
    }
    getRelativePath(absolutePath) {
        const relativePath = path.relative(this.workingDirectory, absolutePath);
        return relativePath;
    }
    getAbsolutePath(relativePath) {
        const absolutePath = path.join(this.workingDirectory, relativePath);
        return absolutePath;
    }
    setFromContent(content) {
        const rev = this.addContent(content);
        this.fromContentRev = rev;
        this.fromGitRev = undefined;
        this.fromGitWorkingDirectory = undefined;
    }
    setToContent(content) {
        const rev = this.addContent(content);
        this.toContentRev = rev;
        this.toGitRev = undefined;
        this.toGitWorkingDirectory = undefined;
    }
    setFromGitInfo({ rev }) {
        return __awaiter(this, void 0, void 0, function* () {
            const absoluteGitWorkingDirectory = yield this.getAbsoluteGitWorkingDirectory(this.fromAbsolutePath);
            this.fromGitWorkingDirectory =
                this.getRelativePath(absoluteGitWorkingDirectory) || undefined;
            this.fromGitRev = yield this.parseRev(rev, absoluteGitWorkingDirectory);
            this.fromContentRev = undefined;
        });
    }
    setToGitInfo({ rev }) {
        return __awaiter(this, void 0, void 0, function* () {
            const absoluteGitWorkingDirectory = yield this.getAbsoluteGitWorkingDirectory(this.toAbsolutePath);
            this.toGitWorkingDirectory =
                this.getRelativePath(absoluteGitWorkingDirectory) || undefined;
            this.toGitRev = yield this.parseRev(rev, absoluteGitWorkingDirectory);
            this.toContentRev = undefined;
        });
    }
    getAbsoluteGitWorkingDirectory(filePath) {
        return __awaiter(this, void 0, void 0, function* () {
            const [dirname] = getDirnameBasename(filePath);
            const git = simpleGit(dirname);
            const rootDir = yield git.revparse(["--show-toplevel"]);
            const realDirname = yield fse.realpath(dirname);
            const relative = path.relative(realDirname, rootDir);
            const absoluteGitWorkingDirectory = path.join(dirname, relative);
            return absoluteGitWorkingDirectory;
        });
    }
    parseRev(rev, absoluteGitWorkingDirectory) {
        return __awaiter(this, void 0, void 0, function* () {
            const parsedRev = yield GitServer.parseRev(rev, absoluteGitWorkingDirectory);
            return parsedRev;
        });
    }
    addContent(content) {
        const rev = sha1(content);
        const filePath = path.join(this.workingDirectory, ".relation", "contents", rev);
        if (!fse.existsSync(filePath)) {
            fse.ensureFileSync(filePath);
            fse.writeFileSync(filePath, content);
        }
        return rev;
    }
    toJSON() {
        const obj = {
            id: this.id,
            fromPath: this.fromPath,
            toPath: this.toPath,
            fromRange: this.fromRange,
            toRange: this.toRange,
            fromGitRev: this.fromGitRev,
            toGitRev: this.toGitRev,
            fromGitWorkingDirectory: this.fromGitWorkingDirectory,
            toGitWorkingDirectory: this.toGitWorkingDirectory,
            fromContentRev: this.fromContentRev,
            toContentRev: this.toContentRev,
        };
        return obj;
    }
    autoSetFromRev({ gitRev } = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const currentGitRev = gitRev || "@";
            const gitContent = yield this.getFromGitContent(currentGitRev);
            const currentContent = this.fromCurrentContent;
            const changes = ChangeServer.singleton().getFixedChanges(gitContent, currentContent);
            const dirty = checkDirty({
                changes: changes,
                range: this.fromRange,
            });
            if (dirty) {
                this.setFromContent(currentContent);
            }
            else {
                yield this.setFromGitInfo({ rev: currentGitRev });
            }
        });
    }
    autoSetToRev({ gitRev } = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const currentGitRev = gitRev || "@";
            const gitHeadContent = yield this.getToGitContent(currentGitRev);
            const currentContent = this.toCurrentContent;
            const changes = ChangeServer.singleton().getFixedChanges(gitHeadContent, currentContent);
            const dirty = checkDirty({
                changes: changes,
                range: this.toRange,
            });
            if (dirty) {
                this.setToContent(currentContent);
            }
            else {
                yield this.setToGitInfo({ rev: currentGitRev });
            }
        });
    }
    getViewerRelationAndContents() {
        return __awaiter(this, void 0, void 0, function* () {
            const originalFromViewerContent = yield this.getFromOriginalContent();
            const originalToViewerContent = yield this.getToOriginalContent();
            const originalFromViewerContentRev = sha1(originalFromViewerContent);
            const originalToViewerContentRev = sha1(originalToViewerContent);
            return [
                {
                    id: this.id,
                    fromRange: this.fromRange,
                    toRange: this.toRange,
                    fromGitRev: this.fromGitRev,
                    toGitRev: this.toGitRev,
                    fromGitWorkingDirectory: this.fromGitWorkingDirectory,
                    toGitWorkingDirectory: this.toGitWorkingDirectory,
                    fromContentRev: this.fromContentRev,
                    toContentRev: this.toContentRev,
                    originalFromViewerContentRev,
                    originalToViewerContentRev,
                },
                {
                    [originalFromViewerContentRev]: originalFromViewerContent,
                    [originalToViewerContentRev]: originalToViewerContent,
                },
            ];
        });
    }
}
export default Relation;
