import { LineRelationTypeEnum } from "../types.js";
export const getLineRelations = (changes) => {
    const lineRelations = [];
    let originalStart = 1;
    let originalEnd = 1;
    let modifiedStart = 1;
    let modifiedEnd = 1;
    for (let i = 0; i < changes.length; i++) {
        const change = changes[i];
        if (change.removed) {
            const nextChange = changes[i + 1];
            originalEnd += change.count;
            let originalRange = [originalStart, originalEnd];
            originalStart = originalEnd;
            let modifiedRange;
            // modified
            if (nextChange === null || nextChange === void 0 ? void 0 : nextChange.added) {
                i++;
                modifiedEnd += nextChange.count;
                modifiedRange = [modifiedStart, modifiedEnd];
                modifiedStart = modifiedEnd;
                lineRelations.push({
                    originalRange,
                    modifiedRange,
                    type: LineRelationTypeEnum.MODIFIED,
                });
                continue;
            }
            modifiedRange = [modifiedStart, modifiedEnd];
            lineRelations.push({
                originalRange,
                modifiedRange,
                type: LineRelationTypeEnum.REMOVED,
            });
            continue;
        }
        if (change.added) {
            modifiedEnd += change.count;
            let modifiedRange = [modifiedStart, modifiedEnd];
            modifiedStart = modifiedEnd;
            let originalRange = [originalStart, originalEnd];
            lineRelations.push({
                originalRange,
                modifiedRange,
                type: LineRelationTypeEnum.ADDED,
            });
            continue;
        }
        originalEnd += change.count;
        const originalRange = [originalStart, originalEnd];
        originalStart = originalEnd;
        modifiedEnd += change.count;
        const modifiedRange = [modifiedStart, modifiedEnd];
        modifiedStart = modifiedEnd;
        lineRelations.push({
            originalRange,
            modifiedRange,
        });
    }
    return lineRelations;
};
